@@ -1,5 +1,6 @@
 print.summary.jointModel <-
-function (x, digits = max(4, getOption("digits") - 4), ...) {
+function (x, digits = max(4, getOption("digits") - 4), 
+        printKnots = FALSE, ...) {
     if (!inherits(x, "summary.jointModel"))
         stop("Use only with 'summary.jointModel' objects.\n")
     cat("\nCall:\n", paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")
@@ -18,8 +19,11 @@ function (x, digits = max(4, getOption("digits") - 4), ...) {
     } else if (x$method == "weibull-PH-GH") {
         cat("Weibull relative risk model\n")
     } else if (x$method == "piecewise-PH-GH") {
-        cat("Relative risk model with piecewise-constant baseline risk function\n\t\t(knots at: ", 
-            paste(round(x$control$knots, 1), collapse = ", "), ")\n", sep = "")
+        if (printKnots)
+            cat("Relative risk model with piecewise-constant baseline risk function\n\t\t(knots at: ", 
+                paste(round(x$control$knots, 1), collapse = ", "), ")\n", sep = "")
+        else 
+            cat("Relative risk model with piecewise-constant baseline risk function\n")
     } else if (x$method == "spline-PH-GH") {
         xx <- if (length(x$control$knots) == 1) {
             kk <- round(unique(x$control$knots[[1]]), 1)
@@ -30,7 +34,10 @@ function (x, digits = max(4, getOption("digits") - 4), ...) {
                 paste(kk[-c(1, length(kk))], collapse = ", ")
             }), sep = ": ", collapse = "\n\t\t")
         }
-        cat("Relative risk model with spline baseline risk function (knots at: ", xx, ")\n", sep = "")
+        if (printKnots)
+            cat("Relative risk model with spline baseline risk function (knots at: ", xx, ")\n", sep = "")
+        else 
+            cat("Relative risk model with spline baseline risk function\n")
     } else {
         cat("log cumulative baseline hazard with B-splines (internal knots at: ", 
             paste(round(exp(x$knots[-c(1, length(x$knots))]), 2), collapse = ", "), ")\n", sep = "")
@@ -88,10 +95,11 @@ function (x, digits = max(4, getOption("digits") - 4), ...) {
         cat("\nScale:", round(exp(x$"CoefTable-Event"[nrow(x$"CoefTable-Event"), 1]), digits), "\n")
     cat("\nIntegration:\n")
     GH <- x$method %in% c("ch-GH", "Cox-PH-GH", "weibull-PH-GH", "weibull-AFT-GH", "piecewise-PH-GH", "spline-PH-GH")
-    cat("method:", if (GH) "Gauss-Hermite" else "Laplace")
+    cat("method:", if (GH && x$control$typeGH == "simple") "Gauss-Hermite" 
+        else if (GH && x$control$typeGH != "simple") "(pseudo) adaptive Gauss-Hermite" else "Laplace")
     if (GH) cat("\nquadrature points:", x$control$GHk, "\n") else cat("\n")
     cat("\nOptimization:\n")
-    cat("Convergence:", if (is.logical(x$conv)) 1 - x$conv else x$conv, "\n")
+    cat("Convergence:", as.numeric(x$conv), "\n")
     cat("\n")
     invisible(x)    
 }
